import { ExecutionContextFlow, ExplicitCallable, Pipeline } from './pipe';
export interface IContext<CallValue = unknown, PreviousValue = unknown> {
    /**
     * The value used to call the pipeline
     * @type {CallValue}
     * @memberof IContext
     */
    readonly callValue: CallValue;
    /**
     * The context execution flow can be `sync` or `async`, based on
     * the invokation of sync methods like `resolveSync` and `toSyncFunction`
     * @type {ExecutionContextFlow}
     * @memberof IContext
     */
    readonly executionFlow: ExecutionContextFlow;
    /**
     * Current mutation index.
     * @type {number}
     * @memberof IContext
     */
    readonly mutationIndex: number;
    /**
     * Previous mutated value.
     * @type {PreviousValue}
     * @memberof IContext
     */
    readonly previousValue: PreviousValue;
    /**
     * An array including previous values
     * @type {unknown[]}
     * @memberof IContext
     */
    readonly previousValues: unknown[];
    /**
     * Calls a function with the `previousValue` and `context` as arguments
     * @template ReturnType
     * @param {ExplicitCallable<PreviousValue, ReturnType>} callable
     * @returns {ReturnType}
     * @memberof IContext
     */
    call<ReturnType>(callable: ExplicitCallable<PreviousValue, ReturnType>): ReturnType;
    /**
     * Applies a mutation.
     * @param {Pipeline} pipeline
     * @returns {IMutatedContext}
     * @memberof IContext
     */
    mutate?(pipeline: Pipeline): IMutatedContext;
}
export interface IMutatedContext {
    /**
     * A list of new mutations to append after the current one has been applied.
     * @type {Pipeline}
     * @memberof IMutatedContext
     */
    pipeline: Pipeline;
}
export declare function applyMutation(context: IContext, pipeline: Pipeline): [IContext, Pipeline];
/**
 * @export
 * @template CallValue
 * @param {CallValue} callValue
 * @param {ExecutionContextFlow} executionFlow
 * @returns {IContext<CallValue>}
 */
export declare function create<CallValue>(callValue: CallValue, executionFlow: ExecutionContextFlow): IContext<CallValue>;
/**
 *
 * @export
 * @template CallValue
 * @template PreviousValue
 * @param {IContext<CallValue>} context
 * @returns {IContext<CallValue, PreviousValue>}
 */
export declare function downgrade<CallValue, PreviousValue = unknown>(context: IContext<CallValue>): IContext<CallValue, PreviousValue>;
/**
 * @export
 * @template CallValue
 * @template PreviousValue
 * @param {IContext<CallValue>} context
 * @param {PreviousValue} previousValue
 * @returns {IContext<CallValue, PreviousValue>}
 */
export declare function update<CallValue, PreviousValue = unknown>(context: IContext<CallValue>, previousValue: PreviousValue): IContext<CallValue, PreviousValue>;
