import { IContext } from './context';
export declare type Callable = <Input, Output>(input: Input, context: IContext<unknown, Input>) => Output;
export declare type ExecutionContextFlow = 'async' | 'sync';
export declare type ExplicitCallable<Input, Output> = (input: Input, context: IContext<unknown, Input>) => Output;
export declare type ResolvedPipe<Input, Output> = (input: Input) => Promise<Output>;
export declare type ResolvedSyncPipe<Input, Output> = (input: Input) => Output;
export declare type PassThroughCallable = <Input>(input: Input, context: IContext<unknown, Input>) => Input;
export declare type Pipeline = (Callable | ExplicitCallable<unknown, unknown>)[];
export interface IPipe<InitialCallable, Output> {
    /**
     * Determines if the function is memoized or not
     * @type {boolean}
     * @memberof IPipe
     */
    readonly memoized: boolean;
    /**
     * @type {'pipe'}
     * @memberof IPipe
     */
    readonly type: 'pipe';
    /**
     * Runs until next breakpoint
     * @returns {IIterablePipe<InitialCallable, Output>}
     * @memberof IPipe
     */
    iter(callValue: InitialCallable): IIterablePipe<InitialCallable, Output>;
    /**
     * Method for chaining a function
     * @template NextValue
     * @param {ExplicitCallable<Output, NextValue>} callable
     * @returns {IPipe<InitialCallable, NextValue>}
     * @memberof IPipe
     */
    pipe<NextValue>(callable: ExplicitCallable<Output, NextValue>): IPipe<InitialCallable, NextValue>;
    /**
     * Resolves the pipeline asynchronously.
     * @type {ResolvedPipe<InitialCallable, Output>}
     * @memberof IPipe
     */
    resolve: ResolvedPipe<InitialCallable, Output>;
    /**
     * Resolves the pipeline synchronously.
     * @type {ResolvedSyncPipe<InitialCallable, Output>}
     * @memberof IPipe
     */
    resolveSync: ResolvedSyncPipe<InitialCallable, Output>;
    /**
     * Creates an `async` function from the pipeline
     * @returns {(input: InitialCallable) => Promise<Output>}
     * @memberof IPipe
     */
    toFunction(): (input: InitialCallable) => Promise<Output>;
    /**
     * Creates a function from the pipeline
     * @returns {(input: InitialCallable) => Output}
     * @memberof IPipe
     */
    toSyncFunction(): (input: InitialCallable) => Output;
}
export interface IIterablePipe<InitialCallable, Output> {
    hasnext(): boolean;
    hasprev(): boolean;
    next(): IIterablePipe<InitialCallable, Output>;
    prev(): IIterablePipe<InitialCallable, Output>;
    value<Value = unknown>(): Value;
}
/**
 * Creates a new pipeline
 * @export
 * @template CallValue
 * @template NextValue
 * @param {ExplicitCallable<CallValue, NextValue>} callable
 * @param {boolean} [memoized=false]
 * @returns
 */
export declare function pipe<CallValue, NextValue>(callable: ExplicitCallable<CallValue, NextValue>, memoized?: boolean): IPipe<CallValue, NextValue>;
